;===================================================
;               SOLAR ORBITER PAS
;            Process_PAS_EGSE_Flight_FS_Full.pro
;                 Version 4.0
;  Flight SequencercCalibration properties plotter. Processing data of PAS_EGSE_Flight
;  Versions:
; 17 May 2017: V.1.0, Fedorov
;  8 Jun 2017: V.1.1, Fedorov, full arry to complete and analyze
; 22 Feb 2019: V.2.0, Fedorov, CALIPSO. Flight sequencer
; 26 Mar 2019: V.2.1, Fedorov, Total count for Azimuth
; 10 Apr 2019: V.3.0, Fedorov, Read the date from the save conf from Show_PAS_EGSE_Flight_FS_Full.pro
; 23 Jan 2020: V.4.0, Fedorov, an approach to integrate data in several Energy steps
;===================================================

;===================================================
;   INTERFACE
;===================================================
DEV = 'X'
UseGDL = 0

;-----------------------------------------------------
; Options for preliminary measurements
;-----------------------------------------------------
;ToPlot = 'ELEV'      ; Elevation response v.mechanical elevation
;ToPlot = 'AZIM'      ; Azimuthal response for the selected CEMs
ToPlot = 'AzEl_Point'  ; Gravity center of the measurement points
;ToPlot = 'Ener_Elev' ; Energy-Elevation 2D distribution of selected CEM(s)
;ToPlot = 'CNT_EL_AZ'    ; 2D plot count in Elevation - Azim plane. It is just a technological plot
;ToPlot = 'TIME'      ; Averaged cound v. time
    
;=======================================================
;  First calibration with Configuration V3.0 Sequencer 4.0. Full Calibration
;==================================================
; File to  read the variables saved by Show_PAS_EGSE_Flight_FS_Full.pro
;==================================================
;ConfSaveFile = '20190410_2126_save_conf.sav'   

;===================================================
; processing files for calibration report
;===================================================
;CNfile = 'PAS_PFM_CN_array_14_Apr_2019.txt'
;Vfile = 'PAS_PFM_V_array_14_Apr_2019.txt'
;AzFile = 'PAS_PFM_Az_array_14_Apr_2019.txt'
;ElFile = 'PAS_PFM_El_array_14_Apr_2019.txt'
;GfFile = 'PAS_PFM_GF_array_14_Apr_2019.txt'
;Code = 'Processed by Process_PAS_EGSE_Flight_FS_Full.pro.V3.0 : '

;=======================================================
;  Second calibration with Configuration V4.2 Sequencer 5.0.
;==================================================
; File to  read the variables saved by Show_PAS_EGSE_Flight_FS_Full.pro
;==================================================
ConfSaveFile = '20200122_Seq5_step42_save_conf.sav'   

;===================================================
; processing files for calibration report
;===================================================
CNfile = 'PAS_PFM_CN_array_01_Feb_2020.txt'
Vfile = 'PAS_PFM_E_V_array_01_Feb_2020.txt'
AzFile = 'PAS_PFM_Az_array_01_Feb_2020.txt'
ElFile = 'PAS_PFM_El_array_01_Feb_2020.txt'
GfFile = 'PAS_PFM_GF_array_01_Feb_2020.txt'
VFfile = 'PAS_PFM_E_V_full_array_01_Feb_2020.txt'
AzFFile = 'PAS_PFM_Az_full_array_01_Feb_2020.txt'
ElFFile = 'PAS_PFM_El_full_array_01_Feb_2020.txt'
Catalog = 'solo_CAL_catalog_20200202_V01.txt'

;==========================================================
; The saved configuration prepared for the processing
;==========================================================
Code = 'Show_PAS_EGSE_Flight_FS_Full.pro.V3.0+Process_PAS_EGSE_Flight_FS_Full.pro.V4.0'

;==================================================
; Plot Ranges
;==================================================
ElevRange = [-25.0,25.0]; deg
AzRange = [-50.0, 30.0]; deg

GFMax = 0.5e-3
GFRange = [GFMax/100.0, GFMax]


;====================================================
; Constants
;====================================================
Nee = 96
Nel = 9
NCEM = 11 ; Reading matrix dimensions
FILLVAL = -1.0e31
TotalBinLim = 1.0e-6 ; Aperture integratet over E/E0 limit value cm2 eV/eV
;-----------------------------------------------
; CutValue force to remove all apertures below Max(Aperture)*CutValue
; Regular value is 0.3, the tested value is 0.1
CutValue = 0.3  ; 0.1

;=====================================================
; Read the saved data
;=====================================================
;--- to note ElAzimApert = fltarr(NCEM,Nel,ElevRSize,FullAzimSize) 
RESTORE, ConfSaveFile
;ElevWSize = n_elements(ElWindow)

;----------------------------------------------------
; Change settings to plot
;-----------------------------------------------------
;CEM_To_Plot = [0,1,2,3,4,5,6,7,8,9,10]
CEM_To_Plot = [10]

ElWindow = [0,1,2,3,4,5,6,7,8]
;ElWindow = [2]
ElevWSize = n_elements(ElWindow)

;----------------------------------------------------------------
; Processing the data
;----------------------------------------------------------------
StepElev = 0.25 ; deg
StepAzim = 0.25 ; deg
EnergyRes = fltarr(11,9)
;------------------------------------------------------------------------------
; ENERGY RESPONSE TO CORRECT ACCORDING dE/E full data
; NOT USED IN THIS VERSION
;------------------------------------------------------------------------------
;for icem = 0, 10 do begin 
;   EnergyRes(icem,*) = [0.0743316,0.0853961,0.0833167, 0.0931188, 0.0846536,0.0820544,0.0482666,0.03, 0.03]
;endfor

;----------------------------------------------------------------
; integrate  the elevation/CEM effective apertures and calculter the centers
; Notes that the apertures already integrated over the dE/E 
; Calculate also the formal boundaries between the bins
;----------------------------------------------------------------
GF = fltarr(11,9) ;[CEMs,Elevations]
Az0 = fltarr(11,9) ; deg
Azmin = fltarr(11,9) ; deg
Azmax = fltarr(11,9) ; deg
El0 = fltarr(11,9) ; deg
Elmin = fltarr(11,9) ; deg
Elmax = fltarr(11,9) ; deg


ElAzimApert(*,*,0,*) = 0.0
iazmin = intarr(9)
iazmin(*) = 0

for iaz = 0, 10 do begin   ; For all CEMs (azimuth)
    ielmin = 0
	for iel = 0, 8 do begin ;  For all elevations   
	
		;-----------------------------------
		; remove all below 30% of the max
		;-----------------------------------
		OneBinArr = reform(ElAzimApert(iaz,iel,*,*))
		MaxCount3 = max(OneBinArr)*CutValue
		index = where(OneBinArr LT MaxCount3) 
		OneBinArr(index) = 0.0
		
 		;-----------------------------------
		; Integration
		;-----------------------------------
        TotalOneBin = total(OneBinArr)
         
        ;---------------------------------------------------
        ; Debugging print, set if(1) to activate
        ;--------------------------------------------------
        if(0) then begin
        if TotalOneBin GT 1.0e-11 then begin
            newMatrix = REBIN(OneBinArr, n_elements(OneBinArr(*,0))*2, n_elements(OneBinArr(0,*))*2)
            TV, fix(newMatrix/max(newMatrix)*255.0)
            wait, 2
            jpegName = 'ImageTot_N_'+string(iel, format='(I02)')+'_'+string(iaz, format='(I02)')
            ;print, jpegName
            write_jpeg, jpegName , tvrd(0,0,n_elements(newMatrix(*,0))-1,n_elements(newMatrix(0,*))-1)
        endif
        end
            
        print, iel, iaz, TotalOneBin, ielmin
        
        ;---------------------------------------------------------------
        ; Elevation and Azimuth gravitation center calculation
        ;---------------------------------------------------------------
        if TotalOneBin GT TotalBinLim then begin
            for ielr = 0, ElevRSize -1 do begin
                El0(iaz, iel) += total(OneBinArr(ielr,*))*ElRArr(ielr)
                Az0(iaz, iel) += total(OneBinArr(ielr,*)*FullAzArr)
            endfor                
            Az0(iaz, iel) = -Az0(iaz, iel)/TotalOneBin ; TO CORRECT THE azimuth scale in CN
            El0(iaz, iel) = -El0(iaz, iel)/TotalOneBin ; TO CORRECT THE elevation scale in CN
            
            ;---------------------------------------------------------------------
            ; Define min and max of angles
            ;---------------------------------------------------------------------
            ;-------------- Elevations ------------------------------------
            if (iel GT ielmin) then begin
                Delta = (El0(iaz, iel) - El0(iaz,iel-1))/2.0
                if (iel - ielmin) EQ 1 then ElMin(iaz, ielmin) = El0(iaz, ielmin) - Delta
                ElMax(iaz, iel-1) = El0(iaz,iel-1) + Delta
                ElMin(iaz, iel) = El0(iaz, iel) - Delta
                ElMax(iaz, iel) = El0(iaz,iel) + Delta                
            endif
            
            ;-------- For Az each point is individual -------------
            if (iaz GT iazmin(iel)) then begin
               Delta = (Az0(iaz, iel) - Az0(iaz-1,iel))/2.0
               if (iaz - iazmin(iel)) EQ 1 then AzMin(iazmin(iel), iel) = Az0(iazmin(iel), iel) - Delta
               AzMin(iaz, iel) = Az0(iaz, iel) - Delta
               AzMax(iaz-1, iel) = Az0(iaz-1, iel) + Delta
               AzMax(iaz, iel) = Az0(iaz, iel) + Delta
            endif
 
            ;-----------------------------------------------------------------------
            ; Velocity geometrical factor calculation scaled to the real accumulation time
            ; It is sm2 sr (cm/s)/(cm/s) 
            ; Energy  response is already integrated in the ElAzimApert, thus there is 0.5 factor to convert to velocity
            ;-------------------------------------------------------------------------
            StepElevRad = StepElev*(!PI/180.0)
            StepAzimRad = StepAzim*(!PI/180.0)*cos(El0(iaz, iel)*(!PI/180.0))
            GF(iaz, iel) = TotalOneBin*StepElevRad*StepAzimRad*0.5 
            ;print, iel, iaz, TotalOneBin,GF(iaz, iel) 

        endif else begin
            Az0(iaz, iel) = FILLVAL
            El0(iaz, iel) = FILLVAL
            AzMin(iaz, iel) = FILLVAL
            ElMin(iaz, iel) = FILLVAL
            AzMax(iaz, iel) = FILLVAL
            ElMax(iaz, iel) = FILLVAL
            GF(iaz, iel) =  FILLVAL
            ielmin = iel+1
            iazmin(iel) = iaz+1
        endelse	
	endfor ; iaz
endfor ;iel

;--------------------------------------------------------
; Print data for another script
;--------------------------------------------------------
print, 'AZIMITH'
for iel = 0, 8 do begin   ; For all elevations
print, Az0(*, iel), format = '("{",11(F10.2,","),"}")'
endfor ;iel

print
print, 'ELEVATIONS:'
for iel = 0, 8 do begin   ; For all elevations
print, El0(*, iel), format = '("{",11(F10.2,","),"}")'
endfor ;iel


;----------------------------------------------------------------------------
; For the moments calculation take the W and CN
;----------------------------------------------------------------------------
Esteps = double(18565.0*0.943^findgen(96))    ; eV
Vsteps = double(13.8e5*sqrt(Esteps))          ; cm/s for protons
;Wsteps = fltarr(96,11,9)  ; (Energy, Azimuth, elevation)
CN = dblarr(96,11,9)  ; (Energy, Azimuth, elevation)

for iel = 0, 8 do begin   ; For all elevations
	for iaz = 0, 10 do begin ; For all CEMs (azimuth)
                if El0(iaz, iel) GT -100.0 then begin
                cel = cos(El0(iaz, iel)*(!PI/180.0))
		for ie = 0, 95 do begin  ; Energy
                   dW = 0.0302*(double(Vsteps(ie))^3)*(AzMax(iaz, iel)-AzMin(iaz, iel))*(ElMax(iaz, iel)-ElMin(iaz, iel))*(!PI/180.0)^2 * cel
                   CN(ie,iaz,iel) = dW/(double(GF(iaz, iel))*Vsteps(ie)^4.0)
                endfor
                endif else for ie = 0, 95 do CN(ie,iaz,iel) = FillVal;
        endfor
endfor

;-----------------------------------------------------------------------------
; Calculation of avareaged values ot Azimuth and Elevation
;------------------------------------------------------------------------------
ElevCent = fltarr(9)
ElevMin = fltarr(9)
ElevMax = fltarr(9)
for iel = 0, 8 do begin 
	index = where(El0(*,iel) GT -500.0,nindex)
    if nindex GT 0 then ElevCent(iel) = total(El0(index,iel)*GF(index, iel))/total(GF(index, iel))
    if iel GT 0 then Delta = (ElevCent(iel) - ElevCent(iel-1))/2.0
    if iel EQ 1 then begin ElevMin(0) = ElevCent(0) - Delta & ElevMax(0) = ElevCent(0) + Delta & end
    ElevMin(iel) = ElevCent(iel) - Delta
    ElevMax(iel) = ElevCent(iel) + Delta
endfor

AzimCent = fltarr(11)
AzimMin = fltarr(11)
AzimMax = fltarr(11)
for iaz = 0, 10 do begin 
	index = where(Az0(iaz,*) GT -500.0,nindex)
    if nindex GT 0 then AzimCent(iaz) = total(Az0(iaz,index)*GF(iaz,index))/total(GF(iaz,index))
    if iaz GT 0 then Delta = (AzimCent(iaz) - AzimCent(iaz-1))/2.0
    if iaz EQ 1 then begin AzimMin(0) = AzimCent(0) - Delta & AzimMax(0) = AzimCent(0) + Delta & end
    AzimMin(iaz) = AzimCent(iaz) - Delta
    AzimMax(iaz) = AzimCent(iaz) + Delta
endfor

;-----------------------------------------------------------------------------
; Do corresponding files
;-----------------------------------------------------------------------------
   ;-------------------------------------------------------------------------------
   ; CN array
   ;-------------------------------------------------------------------------------
   get_lun, olun
   openw, olun, CNfile
   printf, olun, '# ID : CN Table'
   printf, olun, '# Creaion Date/Time : '+TIMESTAMP(SECOND=0.0,/UTC)
   printf, olun, '# Processing Script : '+Code
   printf, olun, '# Original Data File :'+'20190424_PAS_FS_9el_Az***_***_3keV_Conf4.2.txt'
   printf, olun, '# Units : '
   printf, olun, '# Fillvalue : ', FillVal
   printf, olun, '# Format : Energy #
   printf, olun, '#        : Elevation_Number CN[Energy_# Elevation_#, 0 .. 10]'
   printf, olun, '# ------------------------------------------------------------'
   for ie = 0, 95 do begin
      printf, olun, 95 - ie
      for iel = 0, 8 do begin
	 printf, olun, iel, CN(ie,*,iel), format = '(I5,11E10.3)'
      endfor
   endfor
   close, olun
   free_lun, olun

   ;-------------------------------------------------------------------------------
   ; V-E array
   ;-------------------------------------------------------------------------------
   get_lun, olun
   openw, olun, VFile
   printf, olun, '# ID : Simplified (1D) Energy and Velocity Table'
   printf, olun, '# Creaion Date/Time : '+TIMESTAMP(SECOND=0.0,/UTC)
   printf, olun, '# Processing Script : '+Code
   printf, olun, '# Original Data File :'+'20190424_PAS_FS_9el_Az***_***_3keV_Conf4.2.txt'
   printf, olun, '# Units : eV, km/s, eV'
   printf, olun, '# Fillvalue : ', FillVal
   printf, olun, '# Format : Energy_Number Energy Velocity'
   printf, olun, '# ------------------------------------------------------------'
   for ie = 0, Nee-1 do begin
      printf, olun, 95 - ie, Esteps[ie], Vsteps[ie]*1.0e-5,format = '(I5,2F9.2)'
   endfor
   close, olun
   free_lun, olun

   ;-------------------------------------------------------------------------------
   ; Elevation array
   ;-------------------------------------------------------------------------------
   get_lun, olun
   openw, olun, ElFile
   printf, olun, '# ID : Simplified (1D) Elevation Table'
   printf, olun, '# Creaion Date/Time : '+systime()
   printf, olun, '# Processing Script : '+Code
   printf, olun, '# Original Data File :'+'20190424_PAS_FS_9el_Az***_***_3keV_Conf4.2.txt'
   printf, olun, '# Units : deg'
   printf, olun, '# Fillvalue : ', FillVal
   printf, olun, '# Format : Elevation_Number Elevation ElevMin ElevMax sin(Elevation) cos(Elevation)'
   printf, olun, '# ------------------------------------------------------------'
   for iel = 0, 8 do begin
      printf, olun, iel, ElevCent[iel], ElevMin(iel), ElevMax(iel), sin(ElevCent[iel]/180.0*!PI),cos(ElevCent[iel]/180.0*!PI), format = '(I5,3F8.2,2F8.4)'
   endfor
   close, olun
   free_lun, olun

   ;-------------------------------------------------------------------------------
   ; Azimuthal array
   ;-------------------------------------------------------------------------------
   get_lun, olun
   openw, olun, AzFile
   printf, olun, '# ID : Simplified (1D) Azimuthal Table'
   printf, olun, '# Creaion Date/Time : '+TIMESTAMP(SECOND=0.0,/UTC)
   printf, olun, '# Processing Script : '+Code
   printf, olun, '# Original Data File :'+'20190424_PAS_FS_9el_Az***_***_3keV_Conf4.2.txt'
   printf, olun, '# Units : deg'
   printf, olun, '# Fillvalue : ', FillVal
   printf, olun, '# Format : Azimuth_Number Azimuth AzimMin AzimMax sin(Azimuth) cos(Azimuth)'
   printf, olun, '# ------------------------------------------------------------'
   for iaz = 0, 10 do begin
      printf, olun, iaz, AzimCent[iaz], AzimMin(iaz), AzimMax(iaz), sin(AzimCent[iaz]/180.0*!PI),cos(AzimCent[iaz]/180.0*!PI), format = '(I5,3F8.2,2F8.4)'
   endfor
   close, olun
   free_lun, olun
   
   ;-------------------------------------------------------------------------------
   ; Full Elevation array
   ;-------------------------------------------------------------------------------
   get_lun, olun
   openw, olun, ElFFile
   printf, olun, '# ID : Full Elevation Table'
   printf, olun, '# Creaion Date/Time : '+TIMESTAMP(SECOND=0.0,/UTC)
   printf, olun, '# Processing Script : '+Code
   printf, olun, '# Original Data File :'+'20190424_PAS_FS_9el_Az***_***_3keV_Conf4.2.txt'
   printf, olun, '# Units : deg'
   printf, olun, '# Fillvalue : ', FillVal
   printf, olun, '# Format : Azimuth #'
   printf, olun, '#        : Elevation_# Elevation ElevMin ElevMax sin(Elevation) cos(Elevation)'
   printf, olun, '# ------------------------------------------------------------'
   for iaz = 0, 10 do begin
      printf,olun, 'Azimuth # ', iaz
   for iel = 0, 8 do begin
      if El0(iaz, iel) GT -100.0 then $
      printf, olun, iel, El0(iaz, iel), ElMin(iaz, iel), ElMax(iaz, iel), sin(El0(iaz, iel)/180.0*!PI),cos(El0(iaz, iel)/180.0*!PI), format = '(I5,3F9.2,2F8.4)' $
      else printf, olun, iel, FillVal, FillVal, FillVal, 0.0, 0.0, format = '(I5,3G9.2,2F8.4)'
   endfor
   endfor
   close, olun
   free_lun, olun
 
   ;-------------------------------------------------------------------------------
   ; Full Azimuthal array
   ;-------------------------------------------------------------------------------
   get_lun, olun
   openw, olun, AzFFile
   printf, olun, '# ID : Full Azimuth Table'
   printf, olun, '# Creaion Date/Time : '+TIMESTAMP(SECOND=0.0,/UTC)
   printf, olun, '# Processing Script : '+Code
   printf, olun, '# Original Data File :'+'20190424_PAS_FS_9el_Az***_***_3keV_Conf4.2.txt'
   printf, olun, '# Units : deg'
   printf, olun, '# Fillvalue : ', FillVal
   printf, olun, '# Format : Elevation #'
   printf, olun, '#        : Azimuth_#  Azimuth AzimMin AzimMax sin(Azimuth) cos(Azimuth)'
   printf, olun, '# ------------------------------------------------------------'
   for iel = 0, 8 do begin
      printf,olun, 'Elevation # ', iel
   for iaz = 0, 10 do begin
      if Az0(iaz, iel) GT -100.0 then $
      printf, olun, iaz, Az0(iaz, iel), AzMin(iaz, iel), AzMax(iaz, iel), sin(Az0(iaz, iel)/180.0*!PI),cos(Az0(iaz, iel)/180.0*!PI), format = '(I5,3F9.2,2F8.4)' $
      else printf, olun, iaz, FillVal, FillVal, FillVal, 0.0, 0.0, format = '(I5,3G9.2,2F8.4)'  
   endfor
   endfor
   close, olun
   free_lun, olun
   
   ;-------------------------------------------------------------------------------
   ; Full E V array (To Be Modified after in-flight calibration)
   ;-------------------------------------------------------------------------------
   OneElVarr = fltarr(11)
   OneElEarr = fltarr(11)
   openw, olun, VFFile
   printf, olun, '# ID : Full Energy and Velocity Table'
   printf, olun, '# Creaion Date/Time : '+TIMESTAMP(SECOND=0.0,/UTC)
   printf, olun, '# Processing Script : '+Code
   printf, olun, '# Original Data File :'+'20190424_PAS_FS_9el_Az***_***_3keV_Conf4.2.txt'
   printf, olun, '# Units : eV, km/s'
   printf, olun, '# Fillvalue : ', FillVal
   printf, olun, '# Format:'
   printf, olun, '# Energy_Number'
   printf, olun, '# Elevation_Number E[Energy_Number Elevation_Number, 0 .. 10], V[Energy_Number Elevation_Number, 0 .. 10]
   printf, olun, '# ------------------------------------------------------------'
   for ie = 0, Nee-1 do begin
      printf, olun, 95 - ie
      for iel = 0, 8 do begin
        OneElVarr(*) = Vsteps[ie]*1.0e-5
        OneElEArr(*) = Esteps(ie)
        printf, olun, iel,  OneElVarr, OneElVarr, format = '(I5,22F9.2)'
      endfor
   endfor
   close, olun
   free_lun, olun
   
   
   ;--------------------------------------------------------------------------------
   ; Velocity geometrical factor array
   ;--------------------------------------------------------------------------------
   get_lun, olun
   openw, olun, GfFile
   printf, olun, '# ID : Velocity GF*accumulation_Time Table'
   printf, olun, '# Creaion Date/Time : '+TIMESTAMP(SECOND=0.0,/UTC)
   printf, olun, '# Processing Script : '+Code
   printf, olun, '# Original Data File :'+'20190424_PAS_FS_9el_Az***_***_3keV_Conf4.2.txt'
   printf, olun, '# Units : cm^2 sr (cm/s)/(cm/s) s'
   printf, olun, '# Fillvalue : -1.0e+31'
   printf, olun, '# Format : Elevation_Number GFvt[0 .. 10]'
   printf, olun, '# ------------------------------------------------------------'
   for iel = 0, 8 do begin
	 printf, olun, iel, GF(*,iel), format = '(I5,11E11.3)'
   endfor
   close, olun
   free_lun, olun

   ;--------------------------------------------------------------------------------
   ; Calibration data catalog generator
   ;--------------------------------------------------------------------------------
   get_lun, olun
   openw, olun, Catalog
   printf, olun, '##'
   printf, olun, '# Filename : '+Catalog
   printf, olun, '#'
   printf, olun, '# SolarOrbiter SWA-PAS calibration catalog file'
   printf, olun, '#'
   printf, olun, '# This file describe the various calibration files used for SWA-PAS data processing.'
   printf, olun, '#'
   printf, olun, '# Various king of calibration files are used, each one as a start ot validity, '
   printf, olun, '# and its the end of validity is the start of the next one.'
   printf, olun, '#'
   printf, olun, '#'
   printf, olun, '# Start validity	Key	Filename'
   printf, olun, '# --------------	---	--------'
   printf, olun, TIMESTAMP(SECOND=0.0,/UTC), '  EVS ', Vfile 
   printf, olun, TIMESTAMP(SECOND=0.0,/UTC), '  EVF ', VFfile 
   printf, olun, TIMESTAMP(SECOND=0.0,/UTC), '  ELS ', ElFile
   printf, olun, TIMESTAMP(SECOND=0.0,/UTC), '  EVF ', ElFFile
   printf, olun, TIMESTAMP(SECOND=0.0,/UTC), '  AZS ', AzFile
   printf, olun, TIMESTAMP(SECOND=0.0,/UTC), '  AZF ', AzFFile
   printf, olun, TIMESTAMP(SECOND=0.0,/UTC), '  GFF ', GfFile
   printf, olun, TIMESTAMP(SECOND=0.0,/UTC), '  CNF ', CNFile
   close, olun
   free_lun, olun

   
   
  

;===============================================================================
; END OF TABLES CALCULATION
;===============================================================================


;-------------------------------------------------------------------
; Plot the data
;------------------------------------------------------------------
XPSsize = 18.0
XXsize = 1100
case ToPlot of
   'ELEV':        begin & Yscale = 0.6 & XPSsize = 16.0 & end
   'AZIM':        begin & Yscale = 0.6 & XPSsize = 16.0 & end
   'Ener_Elev':   begin & Yscale = 0.6 & XPSsize = 10.0 & end
   'AzEl_Point' : begin & Yscale = float(n_elements(ElAzimApert(0,0,0,*)))/float(n_elements(ElAzimApert(0,0,*,0))) & XPSsize = 10.0 & XXsize = 500 & end
   'CNT_EL_AZ':   begin & Yscale = (AzRange(1) -AzRange(0))/(ElevRange(1) -ElevRange(0)) & XPSsize = 15.0 & XXsize = 700 & end
   'TIME': begin & Yscale = 0.6 & XPSsize = 16.0 & end
endcase

if DEV EQ 'PS' then begin
   set_plot, 'PS'
   if UseGDL then begin
      device, Xsize = XPSsize, Ysize = XPSsize*Yscale,  /color ,yoffset=5, xoffset=0,/ENCAPSULATED
   endif else begin
      device, Xsize = XPSsize, Ysize = XPSsize*Yscale,  /color, bits=8 ,yoffset=5, xoffset=0,SET_FONT='Helvetica',/ENCAPSULATED
   endelse
   LocalChar = 1.0
   LegChar = 0.5
   SymSize = 0.5
   LocalThick=4
   LCharThick = 4
   !P.FONT = 0
endif

if DEV eq 'X' then begin
   set_plot, 'X'
   if UseGDL then begin
      device,  retain=2
   endif else begin
      device, True_Color=24, Decomposed=0, BYPASS_TRANSLATION = 0, retain=2
   endelse
   window,0, xsize=fix(XXsize), ysize=fix(XXsize*Yscale)
   LocalChar = 2.0
   LegChar = 1.0
   SymSize = 1.0
   LocalThick=2
   LCharThick = 1
   !P.FONT = -1
endif

loadct, 38, FILE=getenv('COLORTAB')
DNC = !D.TABLE_SIZE
if UseGDL then begin
   BLACK = 0
endif else begin
   BLACK = DNC -1
endelse
RED =   DNC -4
BLUE =  2
GREEN = (RED-BLUE)/2
WHITE = 0

ColStep = float(RED - BLUE)/float(Nel-1)
ColorArr = fix(findgen(Nel)*ColStep+BLUE)

ColStep = float(RED - BLUE)/float(NCem-1)
ColorCems = fix(findgen(NCem)*ColStep+BLUE)

;-------------- Plot the frame ---------------------------------------
!P.CHARTHICK = LCharThick               ; 'KAZ' - Intergrated over elevation Energy-Azimuth distribution

!P.CHARSIZE = LocalChar
!P.COLOR = BLACK
!X.STYLE = 1
!X.THICK = LocalThick
!Y.STYLE = 1
!Y.THICK = LocalThick

;------------------------------------------------------
; 2D preparation
;------------------------------------------------------
csize = 12

Lev = fltarr(csize)
Leb = intarr(csize)
Leb(*) = 0
Lec = intarr(csize) & Lec(0) = 0

;----- LEVELS --------------------
Lev(csize -1) = GFRange(1)
for i = csize -2, 0, -1 do Lev(i) = Lev(i+1)/2.0
;--------- Colors -----------------
ColStep = float(RED-BLUE)/float(csize-1)
Lecf = findgen(csize -1)*ColStep+float(BLUE)
Lec(1:csize -1) = fix(Lecf)

case ToPlot of
    'ELEV':   begin
             GFRange = GFRange/8.0
             !X.TITLE = 'Elevation, deg'
             !Y.TITLE = 'dGF/dEl'
             plot, ElevRange,GFRange, /nodata, /YLOG
           end
    'AZIM':   begin
             GFRange = GFRange/8.0
             !X.TITLE = 'Azimuth, deg'
             !Y.TITLE = 'dGF/dAz'
             plot, AzRange,GFRange, /nodata, /YLOG
           end
    'AzEl_Point' : begin 
        !Y.TITLE = 'Azimuth, deg'
        !X.TITLE = 'Elevation, deg'
        plot, [ElRArr(0),ElRArr(n_elements(ElRArr)-1)] ,[FullAzArr(0),FullAzArr(n_elements(FullAzArr)-1)], /nodata
        end
   'Ener_Elev':   begin
             !Y.TITLE = 'Elevation, deg'
             !X.TITLE = 'Energy'
             plot, EnergyRange, ElevRange, /nodata
           end
    'CNT_EL_AZ':  begin
             !Y.TITLE = 'Azimuth, deg'
             !X.TITLE = 'Elevation, deg'
             plot, ElevRange, AzRange, /nodata
           end
    'TIME':  begin
             !Y.TITLE = 'Day'
             !X.TITLE = 'Total count ovev Az'
             plot, DayRange, GFRange, /nodata, /YLOG
           end
endcase

case ToPlot of
   'AZIM':  begin
            for iceml = 0,n_elements(CEM_To_Plot)-1 do begin
                icem = CEM_To_Plot(iceml)
                for iel = ElWindow(0), ElWindow(ElevWSize -1) do begin
                   if iel EQ ElWindow(0) then ArrToPlot = reform(ElAzimApert(icem,ElWindow,*,*))*StepElev*(!PI/180.0) *0.5*1.0e3 $
                   else ArrToPlot = ArrToPlot + reform(ElAzimApert(icem,ElWindow,*,*))*StepElev*(!PI/180.0) *0.5*1.0e3 ; cm^2 rad (cm/s)/(cm/s)
                endfor
                if (ElevWSize GT 1) then ArrToPlot = total(ArrToPlot,1); elev Bins
                if(ElevRSize GT 1) then ArrToPlot = total(ArrToPlot,1); mech elevations
                ;print, icem, max(ArrToPlot)
 	        oplot, FullAzArr, ArrToPlot > GFRange(0), thick=LocalThick, color = ColorCems(icem)
            endfor
	   end

   'ELEV': begin
           for iell = 0,n_elements(ElWindow)-1 do begin
                iel = ElWindow(iell)
                FinArrPlot = fltarr(ElevRSize)
                ArrToPlot = reform(ElAzimApert(CEM_To_Plot,iel,*,*))*StepAzim*(!PI/180.0) *0.5*1.0e3; selected Elevation Window
                if(n_elements(CEM_To_Plot) GT 1) then ArrToPlot = total(ArrToPlot,1); add CEMS
                ArrToPlot = total(ArrToPlot,2); add Azimuth
                FinArrPlot += ArrToPlot
                oplot, ElRArr, FinArrPlot > GFRange(0), thick=LocalThick+1, color = ColorArr(iel) ;, linestyle=2
             endfor
	   end
    'AzEl_Point' : begin
            for iel = 0, 8 do begin   ; For all elevations
                for iaz = 0, 10 do begin ; For all CEMs (azimuth)
                  if (Az0(iaz,iel) GT -100.0) AND (El0(iaz,iel) GT -100.0) then begin
                     plots, [-El0(iaz,iel), -Az0(iaz,iel)], psym = 2, color = blue
                     oplot, [-ElMin(iaz,iel), -ElMin(iaz,iel), -ElMax(iaz,iel), -ElMax(iaz,iel), -ElMin(iaz,iel)],$
                            [-Azmin(iaz,iel), -AzMax(iaz,iel), -AzMax(iaz,iel), -Azmin(iaz,iel), -Azmin(iaz,iel)], color = RED
                  endif
                endfor
            endfor
	   end
    'Ener_Elev': begin
        ArrToPlot = total(GFEnerElevArr,3)
	CONTOUR, ArrToPlot, EnerArr,  ElRArr, Levels=lev, C_LABELS = Leb,/FILL, C_COLORS =lec, /overplot
        end

    'CNT_EL_AZ': begin
      if ElevWSize GT 1 then begin
	     if n_elements(CEM_To_Plot) LT 2 then ArrToPlot = total(total(reform(GFOrig(*,ElWindow,*,*,CEM_To_Plot[0])),1),1) $
	     else ArrToPlot = total(total(total(reform(GFOrig(*,ElWindow,*,*,CEM_To_Plot)),1),1),3)
      endif else begin
	     if n_elements(CEM_To_Plot) LT 2 then ArrToPlot = total(reform(GFOrig(*,ElWindow,*,*,CEM_To_Plot[0])),1) $
	     else ArrToPlot = total(total(reform(GFOrig(*,ElWindow,*,*,CEM_To_Plot)),1),3)
      endelse
      
      Lev(csize -1) = max(ArrToPlot)
      for i = csize -2, 0, -1 do Lev(i) = Lev(i+1)/sqrt(10.0)
      ;--------- Colors -----------------
      ColStep = float(RED-BLUE)/float(csize-1)
      Lecf = findgen(csize -1)*ColStep+float(BLUE)
      Lec(1:csize -1) = fix(Lecf)

      CONTOUR, ArrToPlot, ElRArr, AzArr, Levels=lev, C_LABELS = Leb,/FILL, C_COLORS =lec, /overplot
   end

    'TIME': oplot, DayAzim,  AverGFAz > GFRange(0), thick=LocalThick+1, color = BLUE
endcase



if DEV EQ 'PS' then device,/close

end

